@@ -14,7 +14,7 @@ use Filter::signatures;
 use feature 'signatures';
 no warnings 'experimental::signatures';
 
-our $VERSION = '0.17';
+our $VERSION = '0.18';
 
 =head1 NAME
 
@@ -199,11 +199,39 @@ C<output>
 
 Name of the output file
 
+=cut
+
+has output => (
+    is => 'ro',
+);
+
+=item *
+
+C<show_error>
+
+    show_error => 0
+
+Show error message on HTTP errors
+
+=cut
+
+has show_error => (
+    is => 'ro',
+);
+
+=item *
+
+C<fail>
+
+    fail => 1
+
+Let the Perl code C<die> on error
+
 =back
 
 =cut
 
-has output => (
+has fail => (
     is => 'ro',
 );
 
@@ -421,7 +449,9 @@ sub as_lwp_snippet( $self, %options ) {
     $options{ implicit_headers } ||= [];
 
     my @preamble;
+    my @postamble;
     push @preamble, @{ $options{ preamble } } if $options{ preamble };
+    push @postamble, @{ $options{ postamble } } if $options{ postamble };
     my @setup_ua = ('');
 
     my $request_args = join ", ",
@@ -455,10 +485,19 @@ sub as_lwp_snippet( $self, %options ) {
         push @setup_ua, $setup_insecure;
     };
 
+    if( $self->show_error ) {
+        push @postamble,
+            '    die $res->message if $res->is_error;',
+    } elsif( $self->fail ) {
+        push @postamble,
+            '    exit 1 if !$res->{success};',
+    };
+
     @setup_ua = ()
         if @setup_ua == 1;
 
     @preamble = map { "$options{prefix}    $_\n" } @preamble;
+    @postamble = map { "$options{prefix}    $_\n" } @postamble;
     @setup_ua = map { "$options{prefix}    $_\n" } @setup_ua;
 
     return <<SNIPPET;
@@ -472,6 +511,7 @@ sub as_lwp_snippet( $self, %options ) {
         @{[$self->_build_quoted_body()]}
     );
     my \$res = \$ua->request( $request_args );
+@postamble
 SNIPPET
 };
 
@@ -482,7 +522,9 @@ sub as_http_tiny_snippet( $self, %options ) {
     push @{ $options{ implicit_headers }}, 'Host'; # HTTP::Tiny dislikes that header
 
     my @preamble;
+    my @postamble;
     push @preamble, @{ $options{ preamble } } if $options{ preamble };
+    push @postamble, @{ $options{ postamble } } if $options{ postamble };
     my @setup_ua = ('');
 
     my $request_args = join ", ",
@@ -501,6 +543,13 @@ sub as_http_tiny_snippet( $self, %options ) {
     } else {
         push @ssl, verify_SSL => 1;
     };
+    if( $self->show_error ) {
+        push @postamble,
+            '    die $res->{reason} if !$res->{success};',
+    } elsif( $self->fail ) {
+        push @postamble,
+            '    exit 1 if !$res->{success};',
+    };
     my $constructor_args = join ",",
                            $self->_pairlist([
                                      @ssl,
@@ -520,6 +569,7 @@ sub as_http_tiny_snippet( $self, %options ) {
         if @setup_ua == 1;
 
     @preamble = map { "$options{prefix}    $_\n" } @preamble;
+    @postamble = map { "$options{prefix}    $_\n" } @postamble;
     @setup_ua = map { "$options{prefix}    $_\n" } @setup_ua;
 
     my @content = $self->_build_quoted_body();
@@ -539,6 +589,7 @@ sub as_http_tiny_snippet( $self, %options ) {
           @content
         },
     );
+@postamble
 SNIPPET
 };
 
@@ -621,7 +672,7 @@ sub as_curl($self,%options) {
 
     if( my $body = $self->body ) {
         push @request_commands,
-            $options{ long_options } ? '--data' : '-d',
+            $options{ long_options } ? '--data-raw' : '--data-raw',
             $body;
     };
 
